/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.custommerchant;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;

import org.w3c.dom.*;
import org.w3c.dom.traversal.*;

import com.adobe.gl.runtime6.utils.*;
import com.adobe.gl.runtime6.xml.*;

public class CMUtils {
	private CMUtils() {}

	public static File
	getDataSourceFile(GLContext gl, String dstype) {
		final String fname = dstype.endsWith(ECP_SUF) ? dstype
			: dstype+ECP_SUF;
		File dir = new File(
			gl.configFileName(GLUtils.GL_PREFIX+"datasources."));
		gl.runtimeDebugMessage("ds directory " + dir.getAbsolutePath()
			+ " file " + fname);
		return new File(dir, fname);
	}

	/**
	 * Retrieves the list of known datasources.
	 */
	public static String[]
	getDataSources(GLContext gl, boolean all) {
		return gl.getDataSources(ECP_SUF, all);
	}

	public static java.util.Properties
	getProviderInfo(GLContext gl, String provider)
		throws GLException
	{
		return GLParamReader.getData(gl,
			getDataSourceFile(gl, provider));
	}

	public static String
	getShopperID(GLContext gl, Properties pinfo)
		throws GLException
	{
		HttpSession session = gl.getRequest().getSession(true);
		String providerURL = pinfo.getProperty("url");
		String merchantID = pinfo.getProperty("merchant-id");
		String shtag = SHOPPER_ID + providerURL;
		String shid = null;
		synchronized (session) {
			shid = (String) session.getAttribute(shtag);
			if (shid == null)
				session.setAttribute(shtag, shid="");
		}
		synchronized (shid) {
			if ((Object) shid == session.getAttribute(shtag)
			 && "".equals(shid))
			{
				Document doc = null;
				try {
					String query = new StringBuffer(providerURL)
						.append("?action=new-cart&merchant-id=")
						.append(merchantID)
						.toString();
					doc = GLXML.asDom(gl, query);
				}
				catch (FileNotFoundException e) {
					throw new GLException(e.toString());
				}
				String shja = GLXML.asText(doc, "//text()");
				shja = shja.trim();
				/*
				 * NB we change the object SHID points
				 * to here, so synchronized(shid) is
				 * no longer meaningful.
				 */
				session.setAttribute(shtag, shid=shja);
			}
		}
		return shid;
	}

	/* package */
	static void
	fixupPlaceholderReturnURLs(Hashtable a, GLAction action, GLContext gl) {
		ArrayList h = (ArrayList) a.get(" names ");
		if (h==null)
			return;
		a.remove(" names ");
		final int NH = h.size();
		for (int i = 0; i < NH; ++i) {
			String name = (String) h.get(i), rname = null;
			if (name != null) {
				rname = GLUtils.actionParam(name, "success");
				a.put(rname, action.getReturnURL(gl,
						(String) a.get(rname)));
				rname = GLUtils.actionParam(name, "failure");
				a.put(rname, action.getReturnURL(gl, 
						(String) a.get(rname)));
			}
		}
	}
	/* package */
	static void
	setPlaceholderReturnURLs(Hashtable a, String name, String successURL,
		String failURL)
	{
		a.put(GLUtils.actionParam(name, "success"), successURL);
		a.put(GLUtils.actionParam(name, "failure"), failURL);
		ArrayList h = (ArrayList) a.get(" names ");
		if (h == null)
			h = new ArrayList();
		h.add(name);
		a.put(" names ", h);
	}

	public static final String ECP_SUF = ".ecp";

	private static final String SHOPPER_ID =
		GLUtils.GL_PREFIX+"shopper-id.";
};
