/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.custommerchant;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;

import org.w3c.dom.*;
import javax.xml.transform.*;

import com.adobe.gl.runtime6.utils.*;
import com.adobe.gl.runtime6.xml.*;

public class CSWCMOrderForm extends CSWBase {

	/*
	 * -- Constructors.
	 */

	public static CSWCMOrderForm
	newOrderForm(GLContext gl, String block, String provider, String name)
		throws GLException, java.io.FileNotFoundException
	{
		Properties pinfo = CMUtils.getProviderInfo(gl, provider);
		String providerURL = pinfo.getProperty("url");
		String shopperID = CMUtils.getShopperID(gl, pinfo);
		CSWCMOrderForm order =
			new CSWCMOrderForm(gl, name, providerURL,
				pinfo.getProperty("merchant-id"), shopperID,
				provider);
		if (gl.runtimeDebug)
			gl.runtimeDebugMessage(order.toString());
		return order;
	}

	private CSWCMOrderForm(GLContext gl, String name, String url,
		String merchant, String shopper, String provider)
		throws GLException, java.io.FileNotFoundException
	{
		this.merchantID = merchant;
		this.providerURL = url;
		this.shopperID = shopper;
		this.provider = provider;
		this.setName(name);

		String qurl = new StringBuffer(url)
			.append("?action=get-order-form&merchant-id=")
			.append(merchant)
			.append("&shopper-id=").append(shopper)
			.toString();
		this.doc = GLXML.asDom(gl, qurl);
		/*
		 * <custommerchant-result>
		 *	<order-form>
		 * We want to select the (single) <order-form>, so use
		 * 2 levels of *.
		 */
		this.csx = new CSWXML(doc, "/*/*", "x in "+name);
		this.ix = 0;
		this.xix = 0;
		this.blksize = this.nrow = csx.RecordCount();
		this.ncol = 1;	/* ? */
		this.MoveFirst();
	}

	/*
	 * -- Implement the ContentSourceWrapper interface.
	 */

	public String
	Error(String field) {
		return csx.Error(field);
	}

	public java.util.Hashtable
	GetGenerator(boolean b) throws GLException {
		throw new GLException(
			"CSWCMOrderForm: I don't do GetGenerator");
	}
	public String Key() throws GLException {
		throw new GLException("CSWCMOrderForm: I don't do Key");
	}
	public void
	MoveToKey(java.util.Hashtable a) throws GLException {
		throw new GLException("CSWCMOrderForm: I don't do MoveToKey");
	}
	public ContentSourceWrapper
	NestedCSW(String name) throws GLException {
		/*
		 * Lazily update the underlying CSWXML's position.
		 */
		if (xix != ix)
			csx.MoveTo(xix=ix);
		return csx.NestedCSW(name);
	}
	public String[]
	PrimaryKey()
	{
		return new String[0];
	}

	public Object
	Value(int j) throws GLException {
		throw new GLException("Value(z) called out of order");
	}

	public Object
	Value(String name)
		throws GLException
	{
		/*
		 * Check if there's an error value at this position.
		 */
		Object o = errorValue(ix, name);
		if (!GLNull.val.equals(o))
			return o;
		/*
		 * Lazily update the underlying CSWXML's position.
		 */
		if (xix != ix)
			csx.MoveTo(xix=ix);
		return csx.Value(name);
	}

	/*
	 * Other methods.
	 */

	public String
	toString() {
		return new StringBuffer()
			.append("<p><b>CSW ").append(name).append("</b>\r\n")
			.append(super.toString())
			.append("xix ").append(xix).append("<br>\r\n")
			.append("merchantID ").append(merchantID).append("<br>\r\n")
			.append("providerURL ").append(providerURL).append("<br>\r\n")
			.append("shopperID ").append(shopperID).append("<br>\r\n")
			.append("provider ").append(provider).append("<br>\r\n")
			.append("csx {<br>\r\n")
			.append(csx.toString())
			.append("<br>\r\n}<br>\r\n")
			.toString();
	}

	public final String merchantID, providerURL, shopperID, provider;

	/*
	 * -- Private/protected.
	 */

	protected int
	getColumnByName(String field) {
		return 0;
	}

	private Document doc;
	private CSWXML csx;
	private int xix;
}
