/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.jdbc;

import java.lang.*;
import java.util.*;
import com.adobe.gl.runtime6.utils.*;

public class GLDataSourceFactory {

	public static javax.sql.DataSource
	getInstance(javax.servlet.ServletContext appl, String name) {
		GLDataSourceFactory gx =
			(GLDataSourceFactory) appl.getAttribute(FACTNAME);
		GLDataSource ds = gx==null ? null
			: (GLDataSource) gx.dsh.get(name);
		return ds;
	}

	public static javax.sql.DataSource
	getInstance(GLContext gl, String name)
		throws GLException
	{
		GLDataSourceFactory gx = null;
		GLDataSource ds = null;
		javax.servlet.ServletContext appl = gl.getApplication();

		if (logp(10))
			log(10, "GLDataSourceFactory.getInstance "+name+" {");

		/*
		 * Get/init our factory object.
		 */
		synchronized (appl) {
			gx = (GLDataSourceFactory) appl.getAttribute(FACTNAME);
			if (gx==null)
				appl.setAttribute(FACTNAME,
					gx = new GLDataSourceFactory(appl));
		}

		/*
		 * Now find the data source config information and install
		 * it, if need be, into gx.
		 * TODO getDatabaseInfo shouldn't need gl.
		 */
		Hashtable a = getDatabaseInfo(gl, name);
		a.put(DSNAME, name);
		synchronized (gx) {
			ds = (GLDataSource) gx.dsh.get(name);
			if (ds==null
			 || !ds.getSyndrome().equals(
				GLDataSource.makeSyndrome(a)))
			{
				/*
				 * There was an existing data source with
				 * this name.  Shut it down.
				 */
				if (ds != null) {
					gx.dsh.remove(name);
					ds.shutdown();
				}
				/* Make the new one. */
				try {
					ds = new GLDataSource(a);
					gx.dsh.put(name, ds);
				}
				catch (Throwable e) {
					throw new GLException(e.toString());
				}
			}
		}
		log(10, "GLDataSourceFactory.getInstance }");
		return ds;
	}

	private GLDataSourceFactory(javax.servlet.ServletContext appl) {
		this.appl = appl;
	}

	public static Hashtable
	getDatabaseInfo(GLContext gl, String name)
		throws GLException
	{
		return gl.getParams(GLUtils.GL_PREFIX + "datasources." + name);
	}

	public javax.sql.DataSource
	getDataSource(String name) {
		return (javax.sql.DataSource) dsh.get(name);
	}

	public Enumeration
	keys() {
		return dsh.keys();
	}

	/*
	 * -- Private.
	 */

	private static void
	log(int thresh, String msg) {
		if (logLevel >= thresh)
			System.out.println(msg);
	}

	private static boolean
	logp(int thresh) {
		return logLevel >= thresh;
	}

	public static final String FACTNAME = "GLDataSourceFactory0";
	public static final String DSNAME = " my name ";

	private javax.servlet.ServletContext appl;
	private Hashtable dsh = new Hashtable();
	private static final int logLevel = 9;
}
