/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.jdbc;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import com.adobe.gl.runtime6.utils.*;

/**
 * Action on a JDBC database.
 */
public class GLJdbcAction extends GLAction {
	/*
	 * -- Constructor
	 */

	/**
	 * NB CSW must be instanceof CSWResultSet.<br>
	 * NB GL can be slightly misleading: GLJdbcAction objects are built
	 * in the context of a form page, then passed as session data to
	 * an action page.  So we don't actually install GL into our
	 * action object, because it will be invalid during the action page.
	 * On the other hand, we still need to pass GL into the constructor
	 * because it's our handle to the page/request/&c. data we need.
	 */
	protected
	GLJdbcAction(GLContext gl, ContentSourceWrapper csw)
		throws GLException
	{
		/*
		 * Note that we sacrifice column 0, for consistency
		 * with ResultSet.
		 */
		super(gl, csw);
		CSWResultSet csr = (CSWResultSet) csw;
		gl.runtimeDebugMessage(csr.toString());
		String[] a = new String[1+csr.getColumnCount()];
		a[0] = "";
		for (int i = 1; i < a.length; ++i)
			a[i] = csr.getColumnName(i);
		this.columns = a;

		int[] t = new int[this.columns.length];
		t[0] = 0;
		for (int i = 1; i < t.length; ++i)
			t[i] = csr.getColumnType(i);
		this.coltype = t;

		gen = csw.GetGenerator(true);
		/* Set final fields */
		dbname = (String) gen.get("source");
		sql = (String) gen.get("sql");
		key = GLUtils.split(",", (String) gen.get("key"));
		pkey = (String) gen.get("pkey");
		newRecord = csr.newRecord;

		where = (Hashtable) gen.get(key[keyx]);
		if (gl.runtimeDebug) {
			gl.runtimeDebugMessage("<p>");
			gl.runtimeDebugMessage(gen.toString());
			gl.runtimeDebugMessage("<p>key="
				+ (String) gen.get("key"));
			gl.runtimeDebugMessage("<p>key["+keyx+"]='"
				+ key[keyx] + "':" + where);
			debugDump(gl);
		}
	}

	public void
	debugDump(GLContext gl)
	{
		if (!gl.runtimeDebug)
			return;

		StringBuffer bs = gl.runtimeDebugMessageBuffer();
		bs.append(NL)
			.append("<h4>JdbcAction Session Parameters:</h4>\r\n")
			.append(
			 "<table border='1' cellspacing='2' cellpadding='1'>");
		bs.append("<tr><td>dbname</td><td>").append(dbname)
			.append("</td></tr>\r\n");
		bs.append("<tr><td>sql</td><td>").append(sql)
			.append("</td></tr>\r\n");
		bs.append("<tr><td>pkey</td><td>").append(pkey)
			.append("</td></tr>\r\n");
		for (int i = 1; i < columns.length; ++i) {
			bs.append("<tr><td>columns[").append(i)
				.append("]</td><td>")
				.append(columns[i])
				.append("</td></tr>\r\n");
		}
		for (int i = 1; i < key.length; ++i) {
			bs.append("<tr><td>key[")
				.append(i).append("]</td><td>")
				.append(key[i])
				.append("</td></tr>\r\n");
		}
		bs.append("<tr><td>where</td><td>").append(where)
			.append("</td></tr>\r\n");
		bs.append("</table>\r\n");
	}

	/**
	 * Set VALUES.length consecutively-numbered fields, starting
	 * at OFF+1, in PST.
	 */
	public void
	fillClause(PreparedStatement pst, String[] keys,
		Hashtable values, int off)
		throws SQLException
	{
		int n = GLJdbc.numKeys(keys);
		for (int i = 0; i < n; ++i) {
			String k = keys[i];
			Object o = values.get(k);
			if (GLNull.val.equals(o))
				o = null;
			pst.setObject(off+1+i, o, coltype[findcol(k)]);
		}
	}

	/**
	 * Get the data from FORM that matches our column names.
	 * Convert it into objects for PreparedStatement's benefit.
	 */
	public Hashtable
	getFormData(GLContext gl, Hashtable form)
		throws GLException
	{
		return getFormData(gl, form, key[keyx]);
	}

	public Hashtable
	getFormData(GLContext gl, Hashtable form, final String kx)
		throws GLException
	{
		Hashtable h = new Hashtable();
		final int kxn = kx.length();
		gl.runtimeDebugMessage("kx = "+kx);
		for (Enumeration e = form.keys(); e.hasMoreElements();) {
			String k = (String) e.nextElement();
			if (kxn > 0 && !k.endsWith(kx))
				continue;
			/*
			 * Extract the column name from the key.
			 * A key named FOO[1] corresponds to column FOO;
			 * so does a key named ~FOO[1].
			 */
			int lclip = k.startsWith("~") ? 1 : 0;
			String colname = k.substring(lclip, k.length()-kxn);
			int cx = findcol(colname);
			if (cx > 0) {
				String val = gl.getFormValue(form,
					k.substring(lclip));
				h.put(colname,
					GLJdbc.toObject(val, coltype[cx], gl));
			}
		}
		return h;
	}

	public Hashtable
	getFormData(GLContext gl, ContentSourceWrapper csf)
		throws GLException
	{
		Hashtable h = new Hashtable();
		for (int j = 1; j < columns.length; ++j) {
			Object val = csf.Value(columns[j]);
			if (!GLNull.val.equals(val))
				h.put(columns[j],
					GLJdbc.toObject((String) val,
						coltype[j], gl));
		}
		return h;
	}

	public String
	getRowName() {
		return key[keyx];
	}

	/**
	 * Return the name of the table that we'll be acting upon.
	 * This seems naive; what if the originating query was a join?
	 */
	public String
	getTableName()
		throws GLException
	{
		return GLJdbc.getTableName(sql);
	}

	public boolean
	nextRow(GLContext gl)
	{
		if (++keyx >= key.length) {
			where = null;
			return false;
		}
		where = (Hashtable) gen.get(key[keyx]);
		return true;
	}

	/**
	 * @deprecated see {@link #setup(GLContext, ContentSourceWrapper,
	 *	String, String, String)}
	 */
	public static void
	setup(GLContext gl, ContentSourceWrapper csw, boolean allRecords)
		throws GLException
	{
		setup(gl, csw, "Doh", deflink, deflink);
	}
	public static void
	setup(GLContext gl, ContentSourceWrapper csw, String name,
		String successURL, String failURL)
		throws GLException
	{
		CSWResultSet csr = (CSWResultSet) csw;

		/*
		 * Map success/fail URL to site-relative name unless it is
		 * "$return"
		 */
		String updatedSuccessURL = deflink.equals(successURL)
			? successURL
			: getReturnNonDefaultURL(gl, successURL);
		String updatedFailURL = deflink.equals(failURL) ? failURL
			: getReturnNonDefaultURL(gl, failURL);

		csr.envelope.put(GLUtils.actionParam(name, "success"),
			updatedSuccessURL);
		csr.envelope.put(GLUtils.actionParam(name, "failure"),
			updatedFailURL);
	}
	public static void
	setupAdd(GLContext gl, ContentSourceWrapper csw,
		String successURL, String failURL)
		throws GLException
	{
		setupAdd(gl, successURL, failURL, csw);
	}
	public static void
	setupAdd(GLContext gl, String successURL, String failURL,
		ContentSourceWrapper csw)
		throws GLException
	{
		CSWResultSet csr = (CSWResultSet) csw;
		Integer ix = new Integer(csr.AbsolutePosition());
		csr.indices.add(ix);
		csr.envelope.put("$position", ix);
		setup(gl, csw, "add", successURL, failURL);
	}
	public static void
	setupBlank(GLContext gl, String successURL, String failURL,
		ContentSourceWrapper csw)
		throws GLException
	{
		setup(gl, csw, "blank", successURL, failURL);
	}
	public static void
	setupBlank(GLContext gl, ContentSourceWrapper csw,
		String successURL, String failURL)
		throws GLException
	{
		setup(gl, csw, "blank", successURL, failURL);
	}
	public static void
	setupDelete(GLContext gl, ContentSourceWrapper csw,
		String successURL, String failURL)
		throws GLException
	{
		setup(gl, csw, "delete", successURL, failURL);
	}
	public static void
	setupDelete(GLContext gl, String successURL, String failURL,
		ContentSourceWrapper csw)
		throws GLException
	{
		setup(gl, csw, "delete", successURL, failURL);
	}
	public static void
	setupDuplicate(GLContext gl, String successURL, String failURL,
		ContentSourceWrapper csw)
		throws GLException
	{
		setup(gl, csw, "duplicate", successURL, failURL);
	}
	public static void
	setupDuplicate(GLContext gl, ContentSourceWrapper csw,
		String successURL, String failURL)
		throws GLException
	{
		setup(gl, csw, "duplicate", successURL, failURL);
	}
	public static void
	setupForm(GLContext gl, ContentSourceWrapper csw)
		throws GLException
	{
		CSWResultSet csr = (CSWResultSet) csw;
		csr.envelope.put("$csg", gl.intern(new GLJdbcAction(gl, csr)));
		csr.envelope.put("$cswName", csr.GetName());
		gl.writeSetupHtml(csr.envelope);
		csr.envelope.clear();
	}
	public static void
	setupUpdate(GLContext gl, String successURL, String failURL,
		ContentSourceWrapper csw)
		throws GLException
	{
		setup(gl, csw, "update", successURL, failURL);
	}
	public static void
	setupUpdate(GLContext gl, ContentSourceWrapper csw,
		String successURL, String failURL)
		throws GLException
	{
		setup(gl, csw, "update", successURL, failURL);
	}

	public final String[] columns;
	public final int[] coltype;
	public final String[] key;
	public final String dbname, sql, pkey;
	public Hashtable where;
	public final boolean newRecord;

	private int keyx = 0;
	private Hashtable gen;

	private int
	findcol(String cname)
	{
		for (int i = 1; i < columns.length; ++i)
			if (cname.equalsIgnoreCase(columns[i]))
				return i;
		return 0;
	}

	private static final String NL = System.getProperty("line.separator");
};
