/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.lang.*;
import java.net.*;
import java.util.*;
import com.adobe.gl.runtime6.jdbc.CSWResultSet;

/**
 * GLAction is a superclass for data source-specific action classes.
 */
public abstract class GLAction {
	public
	GLAction(GLContext gl, ContentSourceWrapper csw) {
		Hashtable a = (Hashtable) gl.getRequestParameters().clone();
		fixupNavigation(a, csw);
		this.retQuery = a;
		this.retURI = gl.getRequest().getContextPath()
			+ gl.getRequest().getServletPath();
		this.fullReturnURL = buildReturnURL(retURI, retQuery);
		if (csw != null) {
			this.firstRecord = csw.FirstRecordOnPage();
			this.lastRecord = csw.LastRecordOnPage();
		}
		else {
			this.firstRecord = 1;
			this.lastRecord = 1;
		}
	}

	/**
	 * Removes navigation parameters from A.
	 * These include general parameters like FORMERR= and
	 * parameters specific to CSW (e.g., RECORD_INDEX[CSW.name]).
	 */
	private void
	fixupNavigation(Hashtable a, ContentSourceWrapper csw) {
		a.remove("$key");
		a.remove(GLContext.FORMERR_NAME);
		if (csw == null)
			return;

		/* XXX this is just like gl.LinkToARecord */
		String rkname = "RECORD_KEY["+csw.GetName()+"]";
		String rxname = "RECORD_INDEX["+csw.GetName()+"]";
		String rkval = (String) a.get(rkname);
		String rxval = (String) a.get(rxname);
		a.remove(rkname);
		a.remove(rxname);
		String[] pkey = csw.PrimaryKey();
		if (pkey != null)
			for (int i = 0; i < pkey.length; ++i)
				a.remove(pkey[i]);
		boolean isCopy = false;
		if ("$lastRecord".equals(rkval)
		 || (isCopy = "$copyRecord".equals(rkval))
		 || "$newRecord".equals(rkval))
		{
			if (!isCopy) {
				a.put(rkname, rkval);
				return;
			}
			if (csw instanceof CSWResultSet) {
				CSWResultSet csr = (CSWResultSet) csw;
				a.put(rkname, rkval);
				a.put(rxname, ""+csr.copyRecord);
				return;
			}
		}
		a.put(rxname, ""+csw.FirstRecordOnPage());
	}

	/**
	 * Computes a url used to return to this page (or to redirect
	 * to another page, if URL is not "$return".)  URL may be
	 * specified relative to the current page, so we make sure
	 * it's a rooted url here -- otherwise, if the redirecting
	 * page is in a different directory, the relative url would
	 * no longer be valid.
	 */
	public String
	getReturnURL(GLContext gl, String url) {
		if (deflink.equals(url))
			return fullReturnURL;
		url = gl.contextPathFrom(url);
		if (url.indexOf('/') != 0)
			return url;
		return gl.getRequest().getContextPath() + url;
	}

	/**
	 * Like getReturnURL, but doesn't check for "$return".
	 */
	public static String
	getReturnNonDefaultURL(GLContext gl, String url) {
		url = gl.contextPathFrom(url);
		if (url.indexOf('/') != 0)
			return url;
		return gl.getRequest().getContextPath() + url;
	}

	/**
	 * Constructs the return URL.
	 */
	private static String
	buildReturnURL(String retURI, Hashtable retQuery) {
		String q = retQuery==null ? ""
			: GLUtils.queryArrayToString(retQuery);
		if ("".equals(q))
			return retURI;
		return new StringBuffer(retURI).append('?').append(q)
			.toString();
	}

	public final String fullReturnURL;
	public final String retURI;
	public final Hashtable retQuery;
	public static final String deflink = "$return";
	public final int firstRecord, lastRecord;
};
