/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import org.apache.regexp.*;

/**
 * Extra text filters.
 * This class only exists to provide a name space.
 */
public abstract class GLExtraFilters {
	/**
	 * Truncates OS to CUTOFFLENGTH (and adds SUFFIX if OS was too long).
	 */
	public static String
	truncate(GLContext gl, Object os, int cutOffLength, String suffix) {
		String s = GLNull.nonnull(os).toString();
		if (s.length() <= cutOffLength)
			return s;
		if (cutOffLength <= 0)
			return suffix;
		return s.substring(0, cutOffLength) + suffix;
	}

	/**
	 * If OS matches REGEXP, returns MATCH (possibly with $1-$9
	 * substituted), else NOTMATCH.
	 * If MATCH (NOTMATCH) is empty, returns the original string OS
	 * instead.
	 */
	public static String
	regExpMatch(GLContext gl, Object os, String regExp, boolean ignoreCase,
		String match, String notMatch)
	{
		String s = GLNull.nonnull(os).toString();
		RE re = remake(gl, regExp, ignoreCase);
		if (re == null)
			return patternFailure;

		if (!re.match(s))	/* not matched */
			return "".equals(notMatch) ? s : notMatch;

		if ("".equals(match))
			return s;

		/*
		 * Replace ${digit} with match results.
		 */
		if (match.indexOf('$') < 0)	/* nothing to do */
			return match;
		StringBuffer ts = new StringBuffer();
		final int N = match.length();
		for (int i = 0; i < N; ++i) {
			char c = match.charAt(i);
			if (c != '$' || ++i >= N) {
				ts.append(c);
				continue;
			}
			char c2 = match.charAt(i);
			int paren = c2 - '0';
			if (paren < 0 || 9 < paren)
				ts.append('$').append(c2);
			else if (paren < re.getParenCount())
				ts.append(re.getParen(paren));
		}
		return ts.toString();
	}

	/**
	 * Replaces REGEXP in OS with REPLACE and returns the result.
	 */
	public static String
	regExpReplace(GLContext gl, Object os, String regExp,
		boolean ignoreCase, String replace)
	{
		String s = GLNull.nonnull(os).toString();
		RE re = remake(gl, regExp, ignoreCase);
		if (re == null)
			return patternFailure;
		return re.subst(s, replace);
	}

	private static RE
	remake(GLContext gl, String regExp, boolean ignoreCase)
	{
		RE re = null;
		try {
			re = new RE(regExp,
				(ignoreCase ? RE.MATCH_CASEINDEPENDENT
					: RE.MATCH_NORMAL));
		}
		catch (RESyntaxException e) {
			requack(gl, e);
		}
		return re;
	}

	private static void
	requack(GLContext gl, Exception e) {
		e.printStackTrace();
		StringBuffer ts = new StringBuffer();
		GLDebug.reportException(e, ts);
		gl.runtimeDebugMessage(ts.toString());
	}
	private static final String patternFailure = "*Pattern failure*";
};
