/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.util.*;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.xerces.readers.MIME2Java;

import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

/**
 * GL_ParserWrapper mediates between an externally supplied XML parser
 * and GoLive Dynamic Content's server files.  The constructor initializes
 * a parser instance; the parse() method returns a DOM Document reference
 * for traversal or throws an exception if it's invalid.
 *
 * We implement the ErrorHandler interface to gather up for the user
 * any chatty messages the XML parser might produce.
 */
public class GL_ParserWrapper
	implements ErrorHandler
{
	/*
	 * -- Constructor
	 */
	public
	GL_ParserWrapper(GLContext gl) {
		this.gl = gl;
		this.setFeature(
			"http://apache.org/xml/features/"
				+ "dom/defer-node-expansion",
			setDeferredDOM);
		this.setFeature(
			"http://xml.org/sax/features/validation",
			setValidation);
		this.setFeature(
			"http://xml.org/sax/features/namespaces",
			setNameSpaces);
		this.setFeature(
			"http://xml.org/sax/features/schema",
			setSchemaSupport);
		parser.setErrorHandler(this);
	}

	/*
	 * -- Public
	 */
	public Document
	parse(String uri)
		throws Exception
	{
		/* parser.parse(uri); */
		parser.parse(
			new InputSource(
				new FileReader(uri)));
		return parser.getDocument();
	}

	/*
	 * -- interface ErrorHandler
	 */
	public void
	warning(SAXParseException e) {
		gl.runtimeDebugMessage(
			"[Warning] " + getLocationString(e) + ": "
			+ e.getMessage());
	}
	public void
	error(SAXParseException e) {
		gl.runtimeErrorMessage(
			"[Error] " + getLocationString(e) + ": "
			+ e.getMessage());
	}
	public void
	fatalError(SAXParseException e)
		throws SAXException
	{
		gl.runtimeErrorMessage(
			"[Fatal] " + getLocationString(e) + ": "
			+ e.getMessage());
		throw e;
	}

	/*
	 * -- Private
	 */
	private String
	getLocationString(SAXParseException e) {
		StringBuffer ts = new StringBuffer();

		String sysid = GLNull.nonnull(e.getSystemId()).toString();
		/*
		int ix = sysid.lastIndexOf('/');
		if (ix != -1)
			sysid = sysid.substring(ix+1);
		*/
		ts.append(sysid).append(":");

		ts.append(e.getLineNumber()).append(":")
			.append(e.getColumnNumber());
		return ts.toString();
	}
	private void
	setFeature(String feature, boolean state) {
		try {
			parser.setFeature(feature, state);
		}
		catch (SAXNotRecognizedException e) {
			gl.runtimeDebugMessage("Error setting " + feature
				+ " " + state + ": unrecognized.");
		}
		catch (SAXNotSupportedException e) {
			gl.runtimeDebugMessage("Error setting " + feature
				+ " " + state + ": unsupported.");
		}
	}

	private GLContext gl;
	private org.apache.xerces.parsers.DOMParser parser =
		new org.apache.xerces.parsers.DOMParser();

	private static final boolean setValidation = false,
		setNameSpaces = true,
		setSchemaSupport = true,
		setDeferredDOM = true;
};
