<?php
$GLparam = array(
	/*
	 * Arrange to send data in the UTF-8 charset.
	 */
	'output_encoding'=>'UTF-8',
	'content_type'=>'text/xml',
);
require_once("../include/utils.runtime6.php");
require_once("../include/custommerchant.runtime6.php");

/*
 *
 * info/custommerchant.cart.info6.php
 *
 * GoLive design-time support for CustomMerchant shopping carts.
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

/*
 * Return an XML document containing information about the CustomMerchant
 * shopping carts datasources available.  Valid requests are:
 *
 *	custommerchant.cart.info6.php
 *	custommerchant.cart.info6.php?datasources=all
 *	custommerchant.cart.info6.php?datasources=distinct
 *		return a list of e-commerce cart providers
 *		<DATASOURCES>
 *			<DATASOURCE>...
 *
 *	custommerchant.cart.info6.php?provider=adobe
 *		return the schema of a shopping cart from a specific provider
 *		<STRUCTURE>
 *			<FIELD>
 *				<NAME>...
 *
 *	custommerchant.cart.info6.php?getecp=adobe
 *		return raw contents of ecp file
 *
 *	custommerchant.cart.info6.php?putecp=adobe&data=...
 *		write raw contents of ecp file
 *
 *	custommerchant.cart.info6.php?delete=adobe.ecp
 *		delete file
 *
 *	custommerchant.cart.info.php?test=&provider=adobe
 *		<TESTRESULT>
 *			<RECORD>
 *				<FIELD COLUMN='name'>value</FIELD>
 *
 * Note the asymmetry: delete wants the name to end in .ecp,
 * put and get do not.
 */

function theader() {
	setContentTypeHeader("text/plain");
}

function xheader() {
	setContentTypeHeader("text/xml");
}

function WriteProviders($all)
{
	$handle = opendir(GetDataSourcePath());

	xheader();
	echo "<DATASOURCES>\r\n";
	while ($file = readdir($handle)) {
		if (ereg('(.*)\.ecp$', $file, $regs)) {
			echo "\t<DATASOURCE>",
				($all ? $file : $regs[1]),
				"</DATASOURCE>\r\n";
		}
	}
	echo "</DATASOURCES>\r\n";
}

function WriteCartSchema($provider)
{
	xheader();
	$providerInfo = GetProviderInfo($provider);
	$query = "action=get-cart-schema&merchant-id=" . $providerInfo['merchant-id'];
	$xml = GetHTTPResultsAsText($providerInfo['url'], $query);
	if (ereg("<custommerchant-result>(.*)</custommerchant-result>", $xml, $regs)) {
		echo trim($regs[1]);
	} else {
		echo $xml;	// should be an error message
	}
}

/*
 * Play back the contents of the ECP file named by $GETECP.
 */
function GetECP($getecp)
{
	theader();
	$filePath = GetDataSourcePath() . "$getecp.ecp";
	if (!file_exists($filePath) || !is_file($filePath))
		return null;
	readfile($filePath);
}

/*
 * Store $DATA into an ECP file named by $PUTECP.
 * Canonicalize line endings.
 */
function PutECP($putecp, $data)
{
	$filePath = GetDataSourcePath() . "$putecp.ecp";
	$fp = fopen($filePath, "w"); 
	fwrite($fp, ereg_replace("\r\n?|\n", "\r\n", $data));
	fclose($fp);
}


/*
 * Delete file $DELETE.
 */
function Delete($delete)
{
	$filePath = GetDataSourcePath() . $delete;
	if (!file_exists($filePath) || !is_file($filePath))
		return null;
	unlink($filePath);
}

/*
 * Send query to provider, run results through a transform to make them 
 * palatable to GoLive, echo the finished product.  We can fail sending the 
 * message, getting the results, or transforming them; in all cases you should 
 * see the standard <ERROR Number='x' Description='y'/> message.
 */

function TestMe($provider) {
	$providerInfo = GetProviderInfo($provider);
	$query = "action=get-cart&shopper-id=test&merchant-id=" . $providerInfo['merchant-id'];
	$xml = GetHTTPResultsAsText($providerInfo['url'], $query);

	xheader();
	if (indexof($xml, "<ERROR")==0) {
		echo $xml;
		return;
	}

	$sablotron = xslt_create();
	$result = xslt_process($sablotron, 'arg:/_xml', 'file://' . getcwd() . '/custommerchant.jxform.info6.xsl', NULL, array('/_xml' => $xml));
	if ($result)
		echo $result;
	else
		writeErrorMessage(xslt_errno($sablotron), xslt_error($sablotron));
	xslt_free($sablotron);
}

/**
 ** DISPATCH
 **/

RejectUnauthorizedCallers();

$provider	= GetFormValue("provider");
$datasources= GetFormValue("datasources");
$getecp		= GetFormValue("getecp");
$putecp		= GetFormValue("putecp");
$data		= GetFormValue("data");
$delete		= GetFormValue("delete");
$test		= GetFormValue("test");

if ($getecp != "_no_form_value") {
	GetECP($getecp);
} else if ($putecp != "_no_form_value") {
	PutECP($putecp, $data);
	if ($delete != "_no_form_value") {
		Delete($delete);
	}
} else if ($delete != "_no_form_value") {
	Delete($delete);
} else if ($test != "_no_form_value") {
	TestMe($provider);
} else if ($provider != "_no_form_value") {
	WriteCartSchema($provider);
} else {
	WriteProviders($datasources == "all");
}

?>
