<?php

/*
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

$GLparam = array(
	/*
	 * Prevent utils.runtime6.php from calling session_start(),
	 * because it tends to cause problems when PHP is
	 * misconfigured.
	 */
	'no_session'=>1,
	/*
	 * Arrange to send data in the UTF-8 charset.
	 */
	'output_encoding'=>'UTF-8',
	'content_type'=>'text/html',
);
require_once("../include/utils.runtime6.php");
require_once("troubleshootinginclude.php");
?>
<html>
	<head>
		<meta http-equiv="content-type" content="text/html;charset=iso-8859-1">
		<title><?php echo $title; ?></title>
		<link rel="stylesheet" href="gl.css" type="text/css">
	</head>
	<body>
		<a name="top"></a>
		<h1><?php echo $title; ?></h1>
		<p>This report contains information about the status of your PHP server
		software and the Adobe GoLive files installed on the server.
		Below you will find the results of various tests performed by this
		troubleshooting script.</p>
<p>Please refer to the GoLive User Guide and "readme" file for more information
about installing and configuring your PHP server to work with Adobe GoLive.</p>
<p>Click here to run <a href="mysql_testsql.php">test queries directly to MySQL database servers</a>.</p>
<p><a href="troubleshooting.php">Main Troubleshooting Page</a></p>
<?php

function directoryScan($folder, $level)
{
	global $directoryScanError;
	global $GL_substr;
	global $GL_strpos;

	$stride = 2;	//	string, error message
	$checkStr = Array(
		"Empty_Content_Source",	"Content source is not set properly.",
		"Empty_Field",			"Field is not set properly.",
		"EMPTY_VALUE",			"Empty value."
	);

	$msg = "";
	$msg .= "<p>$folder<br>\n";

	$dir = opendir($folder);
	$children = array();
	while ($file = readdir($dir)) {
		if ($file != '.' && $file != '..')
			$children[] = $file;
	}
	closedir($dir);
	usort($children, 'fnamecmp');

	$msg .= "<table border=\"1\" cellpadding=\"4\">\n";
	$msg .= "<tr><th>Filename</th><th>Scan Result</th><th>Lines</th></tr>\n";
	for ($c = 0; $c < count($children); ++$c) {
		$file = $children[$c];
		if (strtolower($GL_substr($file, -4)) != '.php') {
			continue;
		}
		$fullpath = $folder . "/" . $file;
		if (is_dir($fullpath)) {
		} else {
			$msg .= "<tr><td>$file</td><td>";

			if (file_exists($fullpath)) {
				$fileLines = GL_file($fullpath);
				$lnum = 0;
				$errorOnFile = 0;
				for ($lnum = 0; $lnum < count($fileLines);) {
					$content = $fileLines[$lnum++];
					if ($GL_strpos($content, '!-- Mock Content')) {
						break;	// do not check mock content area
					}
					for ($i = 0; $i < count($checkStr); $i += $stride) {
						$str    = $checkStr[$i];
						$errMsg = $checkStr[$i+1];
						if ($GL_strpos($content, $str)) {
							$msg .= "<span class=\"failed\">$str found at line #$lnum</span><br>\n";
							$msg .= "$errMsg <br>\n";
							$errorOnFile++;
						}
					}
				}
				if ($errorOnFile == 0) {
					$msg .= "<span class=\"passed\">OK</span><br>\n";
				} else {
					$directoryScanError++;
				}
				$msg .= "</td><td align=\"right\">$lnum line(s)<br>\n";
				$msg .= "</td></tr>";
			} else {
				$msg .= "Something wrong";
			}
		}
	}
	$msg .= "</table>\n";
	for ($c = 0; $c < count($children); ++$c) {
		$file = $children[$c];
		if ('config' == strtolower($file)) {
			continue;
		}
		$fullpath = $folder . "/" . $file;
		if (is_dir($fullpath)) {
			$msg .= directoryScan($fullpath, 1+$level);
		}
	}
	return $msg;
}

function checkOutputFiles()
{
	global $directoryScanError;
	$directoryScanError = 0;	// reset

	$test = new Test("Check dynamic (PHP) files");
	$test->description =
		"This test detects some simple errors in dynamic (PHP) files.";

	$test->details = "";
	$thisFile = GetCurrentFile();
	$dirname = dirname($thisFile);
	$configPath = dirname($dirname);
	$dataPath = dirname($configPath);

	$test->details .= directoryScan($dataPath, 0);
	if ($directoryScanError == 0) {
		$test->Pass("No errors detected in dynamic files.");
	} else {
		$test->Fail( "Problem(s) found in $directoryScanError file(s).");
	}
	return $test;
}

$TestResults = Array();

clearstatcache();	//	Clear file system cache

// perform each test and store resulting Test objects in TestResults
if (precheck() == 0) {
	$TestResults[] = checkOutputFiles();	// ToDo: move this test to 2nd troubleshooting page
}

// write out the test summary, followed by the individual test details
// print(reportSummary($TestResults));
reportSummary();
reportDetail();

?>
	</body>
</html>
