<?php
$GLparam = array(
	/*
	 * Arrange to send data in the UTF-8 charset.
	 */
	'output_encoding'=>'UTF-8',
    'internal_encoding' => 'UTF-8',
	'content_type'=>'text/xml',
);
require_once("../include/utils.runtime6.php");

// override header setting in utils
header('Content-Type: text/xml; charset=utf-8');


// *****************************************************************************
//
// info/xml.info6.php
//
// GoLive design-time support for XML.
//
// ADOBE SYSTEMS INCORPORATED
// Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
// 
// NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
// License Agreement, Adobe permits you to reproduce and distribute this 
// file only as an integrated part of a web site created with Adobe 
// GoLive software and only for the purpose of enabling your client to 
// display their web site. All other terms of the Adobe license 
// agreement remain in effect.


// -----------------------------------------------------------------------------
// Return an XML document containing information about the datasources available
// for binding via XML.  Valid requests are:
//
//	xml.info6.php?datasources=<all|distinct>
//		return a list available XML applications
//		ex) .../xml.info6.php?datasources=all
//		ex) .../xml.info6.php?datasources=distinct
//
//	xml.info6.php?dstype=<name>			return the structure of a given datasource
//  xml.info6.php?xml=<name>&xsl=<opt>	return default structure for source
//  xml.info6.php?get=<name>			return the contents of the XDS file
//  xml.info6.php?put=<name>&data=<xml> write the contents of the XDS file
//  xml.info6.php?delete=<name>			delete the XDS file

global $xh;

RejectUnauthorizedCallers();

$xh = xslt_create();
xslt_set_scheme_handlers($xh, array('get_all' => 'get_all'));

// writeHeaderComment(GetCurrentURL().'?'.qatostr($HTTP_GET_VARS));
if (($dstype = GetParam("dstype")) != "") {
	WriteDataSourceSchema($dstype);
} elseif (($get = GetParam("get")) != "") {
    GetRaw($get);
} elseif (($put = GetParam("put")) != "") {
    PutRaw($put, GetParam("data"));
} elseif (($xsl = GetParam("xsl")) != "") {
    TransformAndAnalyze(GetParam("xml"), $xsl);
} elseif (($xml = GetParam("xml")) != "") {
    Analyze($xml);
} elseif (GetParam("datasources") == "all" ) {
	WriteDataSources(true);
} elseif (GetParam("delete") == "") { //this makes this the default if we don't recognize anything
    WriteDataSources(false);
}

xslt_free($xh);

// this can appear together with any of the others
if (($delete = GetParam("delete")) != "") {
    DeleteRaw($delete);
}

// -----------------------------------------------------------------------------
// Scheme handler for Sablotron

function get_all($xh, $scheme, $rest)
{
	if (preg_match('/^http[s]?$/i', $scheme)) {
		list($data, $errno, $errstr) = @GL_curl_get("$scheme:$rest");
		if ($errstr == '')
			return $data;
		// $errstr returns the complete server error msg
		// which is too long to be useful in this context
		die(sprintErrorMessage($errno, "The URL $scheme:$rest could not be retrieved. Perhaps it is invalid."));
	}
	
	die(sprintErrorMessage(E_USER_ERROR, "The URL $scheme:$rest does not begin with http:, https:, or file:."));
}

// -----------------------------------------------------------------------------
// Wrapper for various means of opening URLs.

function get_url($url)
{
	global $GL_str_replace;
	if (preg_match('/^http[s]?:\/\//i', $url)) {
		list($data, $errno, $errstr) = @GL_curl_get($url);
		if ($errstr == '')
			return $data;
		// $errstr returns the complete server error msg
		// which is too long to be useful in this context
		die(sprintErrorMessage($errno, "The URL $url could not be retrieved. Perhaps it is invalid."));
	}
	
	if (strncasecmp($url, 'file://', 7) == 0) {
		$fileName = $GL_str_replace('/', '\\', substr($url, 7));
		$fd = fopen($fileName, "r"); 
		$data = fread($fd, filesize($fileName)); 
		fclose($fd); 
		return $data;
	}
	
	die(sprintErrorMessage(E_USER_ERROR, "The URL $url does not begin with http:, https:, or file:."));
}

// -----------------------------------------------------------------------------
// Treat blank parameters as non-existent

function GetParam($name)
{
	$result = GetFormValue($name);
	if ($result == "_no_form_value") {
		$result = "";
	}
	return $result;
}

/*
 * Report the error result (if any) from the most recent XSLT operation.
 */
function xquack() {
	global $xh;
	writeErrorMessage(xslt_errno($xh), xslt_error($xh));
}

// -----------------------------------------------------------------------------
// Get canonical url for xds file.

function XDSURL($dstype)
{
	global $GL_str_replace;
    $dsFilename = GetDataSourcePath() . $dstype . '.xds';
    return 'file://' . $GL_str_replace('\\', '/', realpath($dsFilename));
}

// -----------------------------------------------------------------------------
// Test the datasource.

function TestDataSource($dstype, $testpath)
{
	global $xh;
	
    // get the datasource file
    $dsFilename = XDSURL($dstype);

	// extract the xml and xsl urls
    $xslDSInfo = '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output omit-xml-declaration="yes" />
	<xsl:template match="/">$xslurl=\'<xsl:value-of select="GL6XMLSOURCE/@xslurl" disable-output-escaping="yes"/>\';$xmlurl=\'<xsl:value-of select="GL6XMLSOURCE/@xmlurl"  disable-output-escaping="yes"/>\';</xsl:template>
	</xsl:stylesheet>';
	
	$variables = xslt_process($xh, $dsFilename, 'arg:/_xsl', NULL, array('/_xsl' => $xslDSInfo));
	if (!$variables) {
		xquack();
        return;
	}
	eval($variables); // sets $xslurl and $xmlurl

    // if we're given an xml url as a parameter, use it instead
    $xmlparam = GetParam("xml");
    if ($xmlparam != "") {
        $xmlurl = (indexof($xmlparam, ":") == -1) ? absolutePath($testpath, $xmlparam) : $xmlparam;
    }

    // get the xml data, transforming it if necessary
	if ($xslurl == "") {
		$xmlData = get_url($xmlurl);
	} else {
		$xmlData = xslt_process($xh, $xmlurl, $xslurl);
		if(!$xmlData) {
			xquack();
		    return;
		}
	}

    // get the first record, transformed according to the datasource file
	$xslData = <<<_XSLDATA
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output omit-xml-declaration="yes" />

	<xsl:param name="dsfile" />

    <xsl:variable name="dsdata" select="document(\$dsfile)" />

    <xsl:variable name="firstRecord">
        <xsl:choose>
            <xsl:when test="\$dsdata/GL6XMLSOURCE[@single = 'true']">
                <xsl:copy-of select="/*[1]" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="/*/*[1]" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

	<xsl:template match="/">
	<TESTRESULT><xsl:text>&#10;</xsl:text>
	    <RECORD number="1"><xsl:text>&#10;</xsl:text>
		    <xsl:apply-templates select="\$dsdata/GL6XMLSOURCE/NODE">
                <xsl:with-param name="element" select="\$firstRecord/*" />
            </xsl:apply-templates>
        </RECORD><xsl:text>&#10;</xsl:text>
	</TESTRESULT><xsl:text>&#10;</xsl:text>
	</xsl:template>

	<!-- ignore NODEs that we don't specifically handle -->
	<xsl:template match="NODE" />

    <xsl:template match="NODE[@type='attribute']">
		<xsl:param name="path" />
        <xsl:param name="element" />
		<FIELD>
			<xsl:attribute name="COLUMN"><xsl:value-of select="\$path" />@<xsl:value-of select="@name" /></xsl:attribute>
            <xsl:value-of select="\$element/@*[name() = current()/@name]" />
		</FIELD><xsl:text>&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="NODE[@type='element']">
		<xsl:param name="path" />
        <xsl:param name="element" />
		<FIELD>
            <xsl:choose>
                <xsl:when test="@name = '[body]'">
                    <xsl:attribute name="COLUMN"><xsl:choose>
                        <xsl:when test="string-length(\$path) = 0">[body]</xsl:when>
                        <xsl:otherwise><xsl:value-of select="substring(\$path, 1, string-length(\$path)-1)" /></xsl:otherwise>
                    </xsl:choose></xsl:attribute>
                    <xsl:copy-of select="\$element/node()" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="COLUMN"><xsl:value-of select="\$path" /><xsl:value-of select="@name" /></xsl:attribute>
                    <xsl:copy-of select="\$element/*[name() = current()/@name]/node()" />
                </xsl:otherwise>
            </xsl:choose>
		</FIELD><xsl:text>&#10;</xsl:text>
	</xsl:template>

	<xsl:template match="NODE[@type='structure']">
		<xsl:param name="path" />
        <xsl:param name="element" />
		<xsl:apply-templates>
			<xsl:with-param name="path"><xsl:value-of select="\$path" /><xsl:value-of select="@name" />/</xsl:with-param>
            <xsl:with-param name="element" select="\$element/*[name() = current()/@name]" />
		</xsl:apply-templates>
	</xsl:template>
</xsl:stylesheet>
_XSLDATA;

	$args = array("/_xml" => $xmlData, "/_xsl" => $xslData);
    $params = array("dsfile" => $dsFilename);
	$result = xslt_process($xh, 'arg:/_xml', 'arg:/_xsl', NULL, $args, $params);
    if ($result) {
		echo $result;
	} else {
		xquack();
    }
}

// -----------------------------------------------------------------------------
// Write out datasource schema as XML.

function WriteDataSourceSchema($dstype)
{
	global $xh;
	
	$testpath = GetParam("test");
	if ($testpath != "") {
		TestDataSource($dstype, $testpath);
        return;
	}
	
	/*
	What the XDS file looks like:
	<GL6XMLSOURCE xslurl="foo.xsl" xmlurl="foo.xml" sample="foobar.xml" single="false">
	    <NODE name="one" type="hidden-attribute" />
	    <NODE name="two" type="hidden-element" />
	    <NODE name="three" type="hidden-element">
		<NODE name="three.one" type="attribute" />
	    </NODE>
	    <NODE name="four" type="attribute" />
	    <NODE name="five" type="element" />
	    <NODE name="six" type="element">
		<NODE name="six.one" type="element" />
	    </NODE>
	    <NODE name="seven" type="structure">
		<NODE name="seven.one" type="attribute" />
	    </NODE>
	    <NODE name="eight" type="rowset">
		<NODE name="eight.one" type="element" />
	    </NODE>
	</GL6XMLSOURCE>
	*/

	$dsfilename = XDSURL($dstype);
	$xslData = <<<_XSLDATA
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output omit-xml-declaration="yes" />

	<!-- ignore NODEs that we don't specifically handle -->
	<xsl:template match="NODE" />

	<xsl:template match="NODE[@type='attribute']">
		<xsl:param name="path" />
		<FIELD>
			<NAME><xsl:value-of select="\$path" />@<xsl:value-of select="@name" /></NAME>
		</FIELD>
	</xsl:template>

	<xsl:template match="NODE[@type='element']">
		<xsl:param name="path" />
		<FIELD>
			<NAME><xsl:value-of select="\$path" /><xsl:choose><xsl:when test="@name = '[body]'">.</xsl:when><xsl:otherwise><xsl:value-of select="@name" /></xsl:otherwise></xsl:choose></NAME>
            <xsl:if test="@name = '[body]'">
                <FRIENDLYNAME><xsl:choose>
                        <xsl:when test="string-length(\$path) = 0">[body]</xsl:when>
                        <xsl:otherwise><xsl:value-of select="substring(\$path, 1, string-length(\$path)-1)" /></xsl:otherwise>
                </xsl:choose></FRIENDLYNAME>
            </xsl:if>
		</FIELD>
	</xsl:template>

	<xsl:template match="NODE[@type='rowset']">
		<xsl:param name="path" />
		<FIELD>
			<NAME><xsl:value-of select="\$path" /><xsl:value-of select="@name" /></NAME>
			<STRUCTURE>
				<xsl:apply-templates />
			</STRUCTURE>
		</FIELD>
	</xsl:template>

	<xsl:template match="NODE[@type='structure']">
		<xsl:param name="path" />
		<xsl:apply-templates>
			<xsl:with-param name="path"><xsl:value-of select="\$path" /><xsl:value-of select="@name" />/</xsl:with-param>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="/">
	<STRUCTURE><xsl:text>&#10;</xsl:text>
		<xsl:apply-templates select="/GL6XMLSOURCE/NODE" />
	</STRUCTURE><xsl:text>&#10;</xsl:text>
	</xsl:template>
</xsl:stylesheet>
_XSLDATA;

	$result = xslt_process($xh, $dsfilename, 'arg:/_xsl', NULL, array('/_xsl' => $xslData));
	if ($result)
		echo $result;
	else
		xquack();
}

// -----------------------------------------------------------------------------
// Write out the list of available datasources.  XML format:
//
//		<DATASOURCES>
//			[<DATASOURCE>...]*

function WriteDataSources($all)
{
	$handle = opendir(GetDataSourcePath());

	echo "<DATASOURCES>\r\n";

	while ($file = readdir($handle)) {
		if (ereg("(.*)\.xds", $file, $regs)) {
			echo "\t<DATASOURCE>",
				($all ? $file : $regs[1]),
				"</DATASOURCE>\r\n";
		}
	}
	closedir($handle);

	echo "</DATASOURCES>\r\n";
}

// -----------------------------------------------------------------------------
// Write out raw datasource file.

function GetRaw($dstype)
{
	readfile(GetDataSourcePath() . $dstype);
}

// -----------------------------------------------------------------------------
// Write into raw datasource file.

function PutRaw($dstype, $data)
{
	$fp = fopen(GetDataSourcePath() . $dstype, "w");
	if (function_exists('mb_internal_encoding')) {
        // on multi-byte systems the data will have already been transcoded to utf-8
	    fwrite($fp, $data);
	} else {
        // on single-byte systems we have to do the transcoding here
        fwrite($fp, utf8_encode($data));
    }
	fclose($fp);
}

// -----------------------------------------------------------------------------
// Delete raw datasource file.

function DeleteRaw($dstype)
{
    unlink(GetDataSourcePath() . $dstype);
}
// -----------------------------------------------------------------------------
// Analyze documents and write out the default datasource document.

function Analyze($xmlurl)
{
    DisplayDoc(get_url($xmlurl));
}

function TransformAndAnalyze($xmlurl, $xslurl)
{
	global $xh;
	$result = xslt_process($xh, $xmlurl, $xslurl);
	if ($result) {
		DisplayDoc($result);
	} else
		xquack();
}

function DisplayDoc($doc)
{
	global $xh;
	
	$transform = <<<_TRANSFORM
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output omit-xml-declaration="yes" />

	<xsl:param name="xslurl" />
	<xsl:param name="xmlurl" />
	<xsl:param name="single" select="'false'" />

	<xsl:template name="print-attributes">
		<xsl:param name="nodeset" />
		<xsl:for-each select="\$nodeset">
			<xsl:if test="generate-id() = generate-id(\$nodeset[name() = name(current())][1])">
				<NODE name="{name()}" type="attribute" /><xsl:text>&#10;</xsl:text>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="print-children">
		<xsl:param name="nodeset" />
		<xsl:for-each select="\$nodeset">
			<xsl:if test="generate-id() = generate-id(\$nodeset[name() = name(current())][1])">
				<NODE name="{name()}" type="element"><xsl:text>&#10;</xsl:text>
					<xsl:call-template name="print-element">
						<xsl:with-param name="nodeset" select="\$nodeset[name() = name(current())]" />
					</xsl:call-template>
				</NODE><xsl:text>&#10;</xsl:text>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="print-element">
		<xsl:param name="nodeset" />
		<!-- this is a hack around a sablotron bug -->
		<xsl:variable name="copy"><xsl:copy-of select="\$nodeset" /></xsl:variable>
		<xsl:choose>
			<xsl:when test="\$copy/*[* | @*]">
				<NODE name="[body]" type="hidden-element" /><xsl:text>&#10;</xsl:text>
				<xsl:call-template name="print-attributes">
					<xsl:with-param name="nodeset" select="\$copy/*/@*" />
				</xsl:call-template>
				<xsl:call-template name="print-children">
					<xsl:with-param name="nodeset" select="\$copy/*/*" />
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<NODE name="[body]" type="element" /><xsl:text>&#10;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="/">
		<GL6XMLSOURCE xslurl="{\$xslurl}" xmlurl="{\$xmlurl}" single="{\$single}">
			<xsl:text>&#10;</xsl:text>
			<xsl:choose>
				<xsl:when test="\$single = 'true'">
					<xsl:call-template name="print-element">
						<xsl:with-param name="nodeset" select="/*" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="print-element">
						<xsl:with-param name="nodeset" select="/*/*" />
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</GL6XMLSOURCE>
	</xsl:template>
</xsl:stylesheet>
_TRANSFORM;

	$xsl = GetParam("xsl");
	$xml = GetParam("xml");
	$single = GetParam("single");

	$args = array("/_xml" => $doc, "/_xsl" => $transform);
	$params = array("xslurl" => $xsl, "xmlurl" => $xml, "single" => $single);

	$result = xslt_process($xh, 'arg:/_xml', 'arg:/_xsl', NULL, $args, $params);
	if ($result)
		echo $result;
	else
		xquack();
}

?>
